@@ -3,8 +3,8 @@ import json
 import numpy as np
 import pandas as pd
 import ipywidgets as widgets
-from pydash import has
-from traitlets import Unicode, default, Dict
+from pydash import has, set_, merge
+from traitlets import Unicode, default, Dict, Bool
 from ipywidgets.embed import embed_minimal_html, dependency_state, embed_data
 from ._version import __version__
 
@@ -19,6 +19,8 @@ class Chart(widgets.DOMWidget):
         data (dict): Data to draw. This dictionary corresponds to the "data" argument of Chart.js.
         kind (str): Type of chart. This string corresponds to the "type" argument of Chart.js.
         options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
+        colorscheme (str, optional): Choose a predefined color scheme to your chart. List of all available colorschemes can be found here: https://nagix.github.io/chartjs-plugin-colorschemes/colorchart.html. Defaults to None.
+        zoom (bool, optional): Allow the user to zoom on the Chart once it is created. Disabled for Doughnut, Pie, PolarArea and Radar Charts. Defaults to True.
     """
 
     _view_name = Unicode('ChartView').tag(sync=True)
@@ -31,14 +33,17 @@ class Chart(widgets.DOMWidget):
     _data = Dict().tag(sync=True)
     _options = Dict().tag(sync=True)
     _type = Unicode().tag(sync=True)
+    _colorscheme = Unicode(allow_none=True).tag(sync=True)
+    _zoom = Bool().tag(sync=True)
 
-    def __init__(self, data: dict, kind: str, options: dict = None, colorscheme: str = None):
+    def __init__(self, data: dict, kind: str, options: dict = None, colorscheme: str = None, zoom: bool = True):
 
         super().__init__()
         self.data = data
         self.kind = kind
         self.options = options if options else {}
         self.colorscheme = colorscheme
+        self.zoom = zoom
 
         # Check user input
         self._validate_input()
@@ -52,6 +57,8 @@ class Chart(widgets.DOMWidget):
         self._options = self.options
         self._data = self.data
         self._type = self.kind
+        self._colorscheme = self.colorscheme
+        self._zoom = self.zoom
 
     @default('layout')
     def _default_layout(self):
@@ -64,45 +71,39 @@ class Chart(widgets.DOMWidget):
         To see more details about this structure, please check the documentation: https://nicohlr.gitlab.io/ipychart/user_guide/usage.html
         """
 
-        msg_data = 'Wrong input format for data argument. See https://nicohlr.gitlab.io/ipychart/ for more details'
-        msg_kind = 'Chart kind must be one of : line, bar, radar, doughnut, polarArea, bubble, horizontalBar, pie. See https://nicohlr.gitlab.io/ipychart/ for more details'
-        msg_options = 'Wrong input format for options argument. See https://nicohlr.gitlab.io/ipychart/ for more details'
-        msg_colorscheme = 'Wrong input format for colorscheme argument. See https://nicohlr.gitlab.io/ipychart/ for more details'
+        msg_format = 'Wrong input format for {} argument. See https://nicohlr.gitlab.io/ipychart/user_guide/usage.html for more details'
+        msg_kind = 'Chart kind must be one of : line, bar, radar, doughnut, polarArea, bubble, horizontalBar, pie. See https://nicohlr.gitlab.io/ipychart/user_guide/charts.html for more details'
 
         # Check data argument
-        assert 'datasets' in self.data, msg_data
-        assert len(self.data['datasets']), msg_data
-        assert ['data' in ds for ds in self.data['datasets']] == [True] * len(self.data['datasets']), msg_data
+        assert 'datasets' in self.data, msg_format.format('data')
+        assert len(self.data['datasets']), msg_format.format('data')
+        assert ['data' in ds for ds in self.data['datasets']] == [True] * len(self.data['datasets']), msg_format.format('data')
         if 'kind' in ['bubble', 'scatter']:
-            for d in self.data['datasets']:
-                assert all(isinstance(x, dict) for x in d['data']), msg_data
-                assert all(k in p for k in ('x', 'y', 'r') for p in self.data), msg_data
-        for d in self.data['datasets']:
-            if 'datalabels' in d:
-                assert isinstance(d['datalabels'], dict), msg_data
+            for dataset in self.data['datasets']:
+                assert all(isinstance(x, dict) for x in dataset['data']), msg_format.format("data['datasets']")
+                assert all(k in p for k in ('x', 'y', 'r') for p in self.data), msg_format.format('data')
+        for dataset in self.data['datasets']:
+            dataset['data'] = dataset['data'].tolist() if isinstance(dataset['data'], pd.Series) else dataset['data']
+            if 'datalabels' in dataset:
+                assert isinstance(dataset['datalabels'], dict), msg_format.format('data')
+        if 'labels' in self.data:
+            self.data['labels'] = self.data['labels'].tolist() if isinstance(self.data['labels'], pd.Series) else self.data['labels']
 
         # Check kind argument
         assert self.kind in ['line', 'bar', 'horizontalBar', 'radar', 'doughnut', 'polarArea', 'bubble', 'pie', 'scatter'], msg_kind
 
         # Check options argument
         if self.options:
-            assert isinstance(self.options, dict), msg_options
-            for key in self.options:
-                assert key in ['legend', 'title', 'tooltips', 'scales', 'scale', 'layout', 'animation', 'hover', 'plugins', 'legendCallback'], msg_options
+            assert isinstance(self.options, dict), msg_format.format('options')
+            assert set(self.options.keys()).issubset(set(['legend', 'title', 'tooltips', 'scales', 'scale', 'layout', 'animation', 'hover', 'plugins', 'legendCallback'])), msg_format.format('options')
 
         # Check colorscheme argument
         if self.colorscheme:
-            assert isinstance(self.colorscheme, str), msg_colorscheme
-            if self.options:
-                self.options.update({'plugins': {'colorschemes': {'scheme': self.colorscheme}}})
-            else:
-                self.options = {'plugins': {'colorschemes': {'scheme': self.colorscheme}}}
+            assert isinstance(self.colorscheme, str), msg_format.format('colorscheme')
 
-        # Pandas series handling
-        for d in self.data['datasets']:
-            d['data'] = d['data'].tolist() if isinstance(d['data'], pd.Series) else d['data']
-        if 'labels' in self.data:
-            self.data['labels'] = self.data['labels'].tolist() if isinstance(self.data['labels'], pd.Series) else self.data['labels']
+        # Check zoom argument
+        assert isinstance(self.zoom, bool), msg_format.format('zoom')
+        self.zoom = False if self.kind in ['radar', 'doughnut', 'polarArea', 'pie'] else self.zoom
 
     def _set_default_options(self):
         """
@@ -111,25 +112,22 @@ class Chart(widgets.DOMWidget):
         """
 
         # Display axis by default only for certain types of chart
-        x_axis_display, y_axis_display = (True, True)
-        if self.kind in ['radar', 'doughnut', 'polarArea', 'pie']:
-            x_axis_display, y_axis_display = (False, False)
+        x_axis_display, y_axis_display = (False,) * 2 if self.kind in ['radar', 'doughnut', 'polarArea', 'pie'] else (True,) * 2
 
-        # Override default scales options from Chart.js if not setted by the user
-        if 'scales' not in self.options and 'scale' not in self.options:
-            if self.kind not in ['radar', 'polarArea']:
-                self.options.update({'scales': {
-                    'yAxes': [{'display': y_axis_display, 'ticks': {'beginAtZero': True, 'display': y_axis_display}}],
-                    'xAxes': [{'display': x_axis_display, 'ticks': {'beginAtZero': True, 'display': x_axis_display}}]
-                }})
-            else:
-                self.options.update({'scale': {'ticks': {'beginAtZero': True}}})
+        # Default Axes options
+        if self.kind not in ['radar', 'polarArea']:
+            default_options = {'scales': {
+                'yAxes': [{'display': y_axis_display, 'ticks': {'beginAtZero': True, 'display': y_axis_display}}],
+                'xAxes': [{'display': x_axis_display, 'ticks': {'beginAtZero': True, 'display': x_axis_display}}]}
+            }
+        else:
+            default_options = {'scale': {'ticks': {'beginAtZero': True}}}
 
-        # Override default legend options from Chart.js if not setted by the user
-        if 'legend' not in self.options:
-            if len(self.data['datasets']) == 1 and self.kind in ['bar', 'line', 'horizontalBar', 'bubble', 'radar', 'scatter']:
-                self.options.update({'legend': False})
+        # Default Legend options
+        if len(self.data['datasets']) == 1 and self.kind in ['bar', 'line', 'horizontalBar', 'bubble', 'radar', 'scatter']:
+            default_options = set_(default_options, 'legend', False)
 
+        self.options = merge(default_options, self.options)
 
     def _set_default_style(self):
         """
@@ -200,7 +198,7 @@ class Chart(widgets.DOMWidget):
 
     def to_html(self, path):
         """
-        This function embed the chart widget into an HTML file dumped at the inputed path location.
+        This function embed the chart widget into an HTML file dumped at the inputted path location.
         To see more details about embeding an ipywidget see: https://ipywidgets.readthedocs.io/en/latest/embedding.html
         """
 
