@@ -1,5 +1,7 @@
 import pandas as pd
 import numpy as np
+from typing import Union
+from pydash import set_, merge
 from pandas.api.types import is_numeric_dtype
 from sklearn.neighbors import KernelDensity
 from sklearn.model_selection import GridSearchCV
@@ -35,74 +37,45 @@ class ChartDataFrame():
             y (str): Column of the dataframe used as datapoints for y Axis.
             hue (str, optional): Grouping variable that will produce points with different colors. Defaults to None.
             options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
+            agg (str, optional): The aggregator used to gather data (ex: 'median' or 'mean'). Defaults to None.
 
         Returns:
-            options (dict): options dictionary ready to be inputed into a Chart class (i.e. match ipychart options format)
+            options (dict): options dictionary ready to be inputted into a Chart class (i.e. match ipychart options format).
         """
 
         agg_label = '' if not agg else ' (' + agg + ')'
 
-        # Set axes labels corresponding to dataframe cols when option is not already set
-        if kind != 'radar':
-            if options:
-                if 'scales' in options.keys():
-                    if 'xAxes' in options['scales'].keys():
-                        if 'scaleLabel' not in options['scales']['xAxes'][0]:
-                            options['scales']['xAxes'][0].update({'scaleLabel': {'display': True, 'labelString': x}})
-                        else:
-                            if 'labelString' not in options['scales']['xAxes'][0]['scaleLabel']:
-                                options['scales']['xAxes'][0]['scaleLabel']['labelString'] = x
-                    else:
-                        options['scales'].update({'xAxes': [{'scaleLabel': {'display': True, 'labelString': x}}]})
-                    if 'yAxes' in options['scales'].keys():
-                        if 'scaleLabel' not in options['scales']['yAxes'][0]:
-                            options['scales']['yAxes'][0].update({'scaleLabel': {'display': True, 'labelString': y + agg_label}})
-                        else:
-                            if 'labelString' not in options['scales']['yAxes'][0]['scaleLabel']:
-                                options['scales']['yAxes'][0]['scaleLabel']['labelString'] = y + agg_label
-                    else:
-                        options['scales'].update({'yAxes': [{'scaleLabel': {'display': True, 'labelString': y + agg_label}}]})
-                else:
-                    options['scales'] = {'xAxes': [{'scaleLabel': {'display': True, 'labelString': x}}],
-                                         'yAxes': [{'scaleLabel': {'display': True, 'labelString': y + agg_label}}]}
-            else:
-                options = {'scales': {'xAxes': [{'scaleLabel': {'display': True, 'labelString': x}}],
-                                      'yAxes': [{'scaleLabel': {'display': True, 'labelString': y + agg_label}}]}}
+        if kind not in ['radar', 'pie', 'polarArea', 'doughnut']:
+
+            default_options = {
+                'scales': {'xAxes': [{'scaleLabel': {'display': True, 'labelString': x}}],
+                           'yAxes': [{'scaleLabel': {'display': True, 'labelString': y + agg_label}}]},
+                'tooltips': {'titleFontSize': 18, 'bodyFontSize': 18, 'enabled': True,
+                             'callbacks': {'title': """function(tooltipItem, data) {return '%s = ' + tooltipItem[0].xLabel;};""" % x,
+                                           'label': """function(tooltipItem, data) {return '%s = ' + tooltipItem.yLabel;};""" % (y + agg_label)}}
+            }
+
         else:
-            if options:
-                if 'legend' in options.keys():
-                    if 'display' not in options['legend'].keys():
-                        options['legend']['display'] = True
-                else:
-                    options['legend'] = {'display': True}
-            else:
-                options = {'legend': {'display': True}}
+            y_value = 'tooltipItem.yLabel' if kind == 'radar' else 'data.datasets[0].data[tooltipItem.index]'
+            default_options = {'legend': {'display': True},
+                               'tooltips': {'titleFontSize': 18, 'bodyFontSize': 18, 'enabled': True,
+                                            'callbacks': {'title': """function(tooltipItem, data) {return '%s = ' + data.labels[tooltipItem[0].index];};""" % x,
+                                                          'label': """function(tooltipItem, data) {console.log(data);return '%s = ' + %s;};""" % (y + agg_label, y_value)}}}
 
         # Set legend prefix if "hue" if activated
         if hue:
-            if options:
-                skip_legend = False
-                if 'legend' in options.keys():
-                    if 'labels' in options['legend'].keys():
-                        if 'generateLabels' in options['legend']['labels']:
-                            skip_legend = True
-                    else:
-                        options['legend'].update({'labels': {}})
-                else:
-                    options['legend'] = {'labels': {}}
-            else:
-                options = {'legend': {}}
+            default_options = set_(default_options, 'legend.labels.generateLabels',
+                                   """function(chart) {let labels = Chart.defaults.global.legend.labels.generateLabels(chart);
+                                   labels.map(label => {label['text'] = "%s" + " = " + label['text']; return label});return labels;};""" % hue)
 
-            if not skip_legend:
-                # Callback function to add preficx before legend labels
-                options['legend']['labels']['generateLabels'] = """function(chart) {
-                        let labels = Chart.defaults.global.legend.labels.generateLabels(chart);
-                        labels.map(label => {label['text'] = "%s" + " = " + label['text']; return label});
-                        return labels;};""" % hue
+            default_options = set_(default_options, 'tooltips.callbacks.title',
+                                   """function(tooltipItem, data) {return '%s = ' + tooltipItem[0].xLabel + ' & ' + '%s = ' + data.datasets[tooltipItem[0].datasetIndex].label;};""" % (x, hue))
+
+        options = merge(default_options, options)
 
         return options
 
-    def _create_chart_data_count(self, x: str, dataset_options: [dict, list] = {}):
+    def _create_chart_data_count(self, x: str, dataset_options: Union[dict, list] = {}):
         """
         This function will prepare all the arguments to create a chart from the input of the user.
         Data are counted before being send to the Chart.
@@ -112,10 +85,10 @@ class ChartDataFrame():
             dataset_options ([dict, list], optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
 
         Returns:
-            data (dict): data dictionary ready to be inputed into a Chart class (i.e. match ipychart data format)
+            data (dict): data dictionary ready to be inputted into a Chart class (i.e. match ipychart data format).
         """
 
-        assert x in self.df.columns
+        assert x in self.df.columns, f'Column {x} not found in dataframe columns'
 
         data = {'datasets': []}
 
@@ -128,7 +101,8 @@ class ChartDataFrame():
 
         return data
 
-    def _create_chart_data_agg(self, kind: str, x: str, y: str, hue: str = None, r: str = None, agg: str = None, dataset_options: [dict, list] = {}):
+    def _create_chart_data_agg(self, kind: str, x: str, y: str, r: str = None, hue: str = None, agg: str = None,
+                               dataset_options: Union[dict, list] = {}):
         """
         This function will prepare all the arguments to create a chart from the input of the user.
         Data are automatically aggregated using the method specified in the "agg" argument before being send to the Chart.
@@ -137,22 +111,22 @@ class ChartDataFrame():
             kind (str): The kind of the chart.
             x (str): Column of the dataframe used as datapoints for x Axis.
             y (str): Column of the dataframe used as datapoints for y Axis.
-            hue (str, optional): Grouping variable that will produce points with different colors. Defaults to None.
             r (str, optional): Column used to define the radius of the bubbles (only for bubble chart). Defaults to None.
+            hue (str, optional): Grouping variable that will produce points with different colors. Defaults to None.
             agg (str, optional): The aggregator used to gather data (ex: 'median' or 'mean'). Defaults to None.
             dataset_options ([dict, list], optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
 
         Returns:
-            data (dict): data dictionary ready to be inputed into a Chart class (i.e. match ipychart data format)
+            data (dict): data dictionary ready to be inputted into a Chart class (i.e. match ipychart data format).
         """
 
-        assert x in self.df.columns
-        assert y in self.df.columns
+        assert x in self.df.columns, f'Column {x} not found in dataframe columns'
+        assert y in self.df.columns, f'Column {y} not found in dataframe columns'
         assert is_numeric_dtype(self.df[y]), 'Please input a numeric columns as y'
         assert kind in ['line', 'bar', 'horizontalBar', 'radar', 'doughnut', 'polarArea', 'bubble', 'pie', 'scatter']
 
         if hue:
-            assert hue in self.df.columns
+            assert hue in self.df.columns, f'Column {hue} not found in dataframe columns'
             assert self.df[hue].nunique() <= 10, 'Too much values for the hue column'
 
         if len(dataset_options):
@@ -168,9 +142,8 @@ class ChartDataFrame():
 
             if hue:
                 # Create one dataset for each unique value of the hue column
-                for i, v in enumerate(self.df[hue].unique()):
+                for i, v in enumerate(sorted(self.df[hue].unique())):
                     if isinstance(dataset_options, list):
-                        print(dataset_options[i])
                         data['datasets'].append({'data': self.df[self.df[hue] == v].groupby(x).agg(agg)[y].round(4).tolist(), 'label': v, **dataset_options[i]})
                     else:
                         data['datasets'].append({'data': self.df[self.df[hue] == v].groupby(x).agg(agg)[y].round(4).tolist(), 'label': v, **dataset_options})
@@ -227,34 +200,36 @@ class ChartDataFrame():
 
         return data
 
-    def count(self, x: str, orient: str = 'v', dataset_options: dict = {}, options: dict = None, colorscheme: str = None):
+    def count(self, x: str, horizontal: bool = False, dataset_options: dict = {}, options: dict = None, colorscheme: str = None,
+              zoom: bool = True):
         """
         Show the counts of observations in each categorical bin using bars.
 
         Args:
             x (str): Column of the dataframe used as datapoints for x Axis.
+            horizontal (bool): draw the bar chart horizontally. Defaults to False.
             dataset_options (dict, optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
             options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None. Defaults to None.
+            zoom (bool, optional): Allow the user to zoom on the Chart once it is created. Defaults to True.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
-        assert orient in ['v', 'h'], "orient argument must be either 'v' (vertical) or 'h' (horizontal)"
-
         data = self._create_chart_data_count(x=x, dataset_options=dataset_options)
 
-        if orient == 'v':
-            options = self._create_chart_options(kind='count', options=options, x=x, y='Count', hue=None)
-        else:
+        if horizontal:
             options = self._create_chart_options(kind='count', options=options, x='Count', y=x, hue=None)
+        else:
+            options = self._create_chart_options(kind='count', options=options, x=x, y='Count', hue=None)
 
-        kind = 'bar' if orient == 'v' else 'horizontalBar'
+        kind = 'horizontalBar' if horizontal else 'bar'
 
-        return Chart(data=data, kind=kind, options=options, colorscheme=colorscheme)
+        return Chart(data=data, kind=kind, options=options, colorscheme=colorscheme, zoom=zoom)
 
-    def dist(self, x: str, bandwidth: [float, str] = 'auto', gridsize: int = 1000, dataset_options: dict = {}, options: dict = None, colorscheme: str = None, **kwargs):
+    def dist(self, x: str, bandwidth: Union[float, str] = 'auto', gridsize: int = 1000, dataset_options: dict = {},
+             options: dict = None, colorscheme: str = None, zoom: bool = True, **kwargs):
         """
         Fit and plot a univariate kernel density estimate on a line chart. This is useful to have a representation of the distribution of the data.
 
@@ -265,9 +240,11 @@ class ChartDataFrame():
             dataset_options (dict, optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
             options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None. Defaults to None.
+            zoom (bool, optional): Allow the user to zoom on the Chart once it is created. Defaults to True.
+            kwargs (optionnal): Other keyword arguments are passed down to scikit-learn's KernelDensity class.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
         assert is_numeric_dtype(self.df[x]), 'Please input a numeric columns as x'
@@ -294,7 +271,7 @@ class ChartDataFrame():
         kde_skl.fit(self.df[x].dropna().to_numpy()[:, np.newaxis])
         pdf = np.exp(kde_skl.score_samples(x_grid[:, np.newaxis]))
 
-        data = {'labels': x_grid.tolist(), 'datasets': [{'data': pdf.tolist(), 'pointRadius': 0}]}
+        data = {'labels': x_grid.tolist(), 'datasets': [{'data': pdf.tolist(), 'pointRadius': 0, **dataset_options}]}
 
         options = self._create_chart_options(kind='count', options=options, x=x, y=f'Density (bandwidth: {bandwidth.round(4)})', hue=None)
 
@@ -311,12 +288,12 @@ class ChartDataFrame():
             if 'callback' not in options['scales']['xAxes'][0]['ticks']:
                 options['scales']['xAxes'][0]['ticks']['callback'] = ticks_format_function
 
-        return Chart(data, 'line', options=options)
+        return Chart(data, 'line', options=options, colorscheme=colorscheme, zoom=zoom)
 
-    def line(self, x: str, y: str, hue: str = None, agg: str = 'mean', dataset_options: [dict, list] = {},
-             options: dict = None, colorscheme: str = None):
+    def line(self, x: str, y: str, hue: str = None, agg: str = 'mean', dataset_options: Union[dict, list] = {},
+             options: dict = None, colorscheme: str = None, zoom: bool = True):
         """
-        A line chart is a way of plotting data points on a line. Often, it is used to show trend data, or the comparison of two data sets.
+        A line chart is a way of plotting data points on a line. Often, it is used to show a trend in the data, or the comparison of two data sets.
 
         Args:
             x (str): Column of the dataframe used as datapoints for x Axis.
@@ -326,44 +303,46 @@ class ChartDataFrame():
             dataset_options ([dict, list], optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
             options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
+            zoom (bool, optional): Allow the user to zoom on the Chart once it is created. Defaults to True.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
         data = self._create_chart_data_agg(kind='line', x=x, y=y, hue=hue, agg=agg, dataset_options=dataset_options)
         options = self._create_chart_options(kind='line', options=options, x=x, y=y, hue=hue, agg=agg)
 
-        return Chart(data=data, kind='line', options=options, colorscheme=colorscheme)
+        return Chart(data=data, kind='line', options=options, colorscheme=colorscheme, zoom=zoom)
 
-    def bar(self, x: str, y: str, hue: str = None, agg: str = 'mean', dataset_options: [dict, list] = {},
-            options: dict = None, colorscheme: str = None, horizontal: bool = False):
+    def bar(self, x: str, y: str, hue: str = None, horizontal: bool = False, agg: str = 'mean', dataset_options: Union[dict, list] = {},
+            options: dict = None, colorscheme: str = None, zoom: bool = True):
         """
-        A bar chart provides a way of showing data values represented as vertical bars. It is sometimes used to show trend data, and the comparison of multiple data sets side by side.
+        A bar chart provides a way of showing data values represented as vertical bars. It is sometimes used to show a trend in the data, and the comparison of multiple data sets side by side.
 
         Args:
             x (str): Column of the dataframe used as datapoints for x Axis.
             y (str): Column of the dataframe used as datapoints for y Axis.
             hue (str, optional): Grouping variable that will produce points with different colors. Defaults to None.
+            horizontal (bool): draw the bar chart horizontally. Defaults to False.
             agg (str, optional): The aggregator used to gather data (ex: 'median' or 'mean'). Defaults to None.
             dataset_options ([dict, list], optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
             options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
-            horizontal (bool): draw the bar chart horizontally. Defaults to False.
+            zoom (bool, optional): Allow the user to zoom on the Chart once it is created. Defaults to True.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
         data = self._create_chart_data_agg(kind='bar', x=x, y=y, hue=hue, agg=agg, dataset_options=dataset_options)
         options = self._create_chart_options(kind='bar', options=options, x=x, y=y, hue=hue, agg=agg)
 
         if horizontal:
-            return Chart(data=data, kind='horizontalBar', options=options, colorscheme=colorscheme)
+            return Chart(data=data, kind='horizontalBar', options=options, colorscheme=colorscheme, zoom=zoom)
         else:
-            return Chart(data=data, kind='bar', options=options, colorscheme=colorscheme)
+            return Chart(data=data, kind='bar', options=options, colorscheme=colorscheme, zoom=zoom)
 
-    def radar(self, x: str, y: str, hue: str = None, agg: str = 'mean', dataset_options: [dict, list] = {},
+    def radar(self, x: str, y: str, hue: str = None, agg: str = 'mean', dataset_options: Union[dict, list] = {},
               options: dict = None, colorscheme: str = None):
         """
         A radar chart is a way of showing multiple data points and the variation between them. They are often useful for comparing the points of two or more different data sets.
@@ -378,7 +357,7 @@ class ChartDataFrame():
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
         data = self._create_chart_data_agg(kind='radar', x=x, y=y, hue=hue, agg=agg, dataset_options=dataset_options)
@@ -400,12 +379,13 @@ class ChartDataFrame():
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
         if y:
             data = self._create_chart_data_agg(kind='doughnut', x=x, y=y, agg=agg, dataset_options=dataset_options)
         else:
             data = self._create_chart_data_count(x=x, dataset_options=dataset_options)
+        options = self._create_chart_options(kind='doughnut', options=options, x=x, y=y, hue=None, agg=agg)
 
         return Chart(data=data, kind='doughnut', options=options, colorscheme=colorscheme)
 
@@ -423,13 +403,14 @@ class ChartDataFrame():
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
         if y:
             data = self._create_chart_data_agg(kind='pie', x=x, y=y, agg=agg, dataset_options=dataset_options)
         else:
             data = self._create_chart_data_count(x=x, dataset_options=dataset_options)
+        options = self._create_chart_options(kind='pie', options=options, x=x, y=y, hue=None, agg=agg)
 
         return Chart(data=data, kind='pie', options=options, colorscheme=colorscheme)
 
@@ -447,17 +428,18 @@ class ChartDataFrame():
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
         if y:
             data = self._create_chart_data_agg(kind='polarArea', x=x, y=y, agg=agg, dataset_options=dataset_options)
         else:
             data = self._create_chart_data_count(x=x, dataset_options=dataset_options)
+        options = self._create_chart_options(kind='polarArea', options=options, x=x, y=y, hue=None, agg=agg)
 
         return Chart(data=data, kind='polarArea', options=options, colorscheme=colorscheme)
 
-    def scatter(self, x: str, y: str, hue: str = None, dataset_options: [dict, list] = {},
-                options: dict = None, colorscheme: str = None):
+    def scatter(self, x: str, y: str, hue: str = None, dataset_options: Union[dict, list] = {},
+                options: dict = None, colorscheme: str = None, zoom: bool = True):
         """
         Scatter charts are based on basic line charts with the x axis changed to a linear axis.
 
@@ -465,24 +447,24 @@ class ChartDataFrame():
             x (str): Column of the dataframe used as datapoints for x Axis.
             y (str): Column of the dataframe used as datapoints for y Axis.
             hue (str, optional): Grouping variable that will produce points with different colors. Defaults to None.
-            agg (str, optional): The aggregator used to gather data (ex: 'median' or 'mean'). Defaults to None.
             dataset_options ([dict, list], optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
             options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
+            zoom (bool, optional): Allow the user to zoom on the Chart once it is created. Defaults to True.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
         data = self._create_chart_data_agg(kind='scatter', x=x, y=y, hue=hue, dataset_options=dataset_options)
         options = self._create_chart_options(kind='scatter', options=options, x=x, y=y, hue=hue)
 
-        return Chart(data=data, kind='scatter', options=options, colorscheme=colorscheme)
+        return Chart(data=data, kind='scatter', options=options, colorscheme=colorscheme, zoom=zoom)
 
-    def bubble(self, x: str, y: str, r: str, hue: str = None, dataset_options: [dict, list] = {},
-               options: dict = None, colorscheme: str = None):
+    def bubble(self, x: str, y: str, r: str, hue: str = None, dataset_options: Union[dict, list] = {},
+               options: dict = None, colorscheme: str = None, zoom: bool = True):
         """
-        A bubble chart is used to display three dimensions of data at the same time.
+        A bubble chart is used to display three-dimension data.
         The location of the bubble is determined by the first two dimensions and the corresponding horizontal and vertical axes.
         The third dimension is represented by the radius of the individual bubbles.
 
@@ -491,16 +473,16 @@ class ChartDataFrame():
             y (str): Column of the dataframe used as datapoints for y Axis.
             r (str, optional): Column of the dataframe used as radius for bubbles.
             hue (str, optional): Grouping variable that will produce points with different colors. Defaults to None.
-            agg (str, optional): The aggregator used to gather data (ex: 'median' or 'mean'). Defaults to None.
             dataset_options ([dict, list], optional): These are options directly related to the dataset object (i.e. options concerning your data). Defaults to {}.
             options (dict, optional): All options to configure the chart. This dictionary corresponds to the "options" argument of Chart.js. Defaults to None.
             colorscheme (str, optional): Colorscheme to use when drawing the chart. Defaults to None.
+            zoom (bool, optional): Allow the user to zoom on the Chart once it is created. Defaults to True.
 
         Returns:
-            [ipychart.Chart]: A chart which display the data using ipychart
+            [ipychart.Chart]: A chart which display the data using ipychart.
         """
 
         data = self._create_chart_data_agg(kind='bubble', x=x, y=y, r=r, hue=hue, dataset_options=dataset_options)
         options = self._create_chart_options(kind='bubble', options=options, x=x, y=y, hue=hue)
 
-        return Chart(data=data, kind='bubble', options=options, colorscheme=colorscheme)
+        return Chart(data=data, kind='bubble', options=options, colorscheme=colorscheme, zoom=zoom)
